/**
 * Defines a generic HashMap (which has the same interface
 * as the Map class, but is backed by a collection of
 * linked lists and is more accessible to us at this point
 * of the course).
 */

#ifndef _hash_map_
#define _hash_map_

template <typename Key, typename Value>
class HashMap {

public:
    HashMap(int sizeHint = 10001);
    ~HashMap();
    
    bool isEmpty() const { return size() == 0; }
    int size() const { return count; }
    
    bool containsKey(const Key& key) const;
    void put(const Key& key, const Value& value);
    Value get(const Key& key) const;
    Value& operator[](const Key& key);
    
private:
    struct node {
        Key key;
        Value value;
        node *next;
    };
    
    node **buckets;
    int numBuckets;
    int count;
    
    int hash(const Key& key) const;
    node *ensureNodeExists(const Key& key);
    const node *findNode(const Key& key) const;
};

#include "hash-map-impl.h"

#endif
