/**
 * File: hash-string.h
 * -------------------
 * Presents a hacky adaptation of hash<const char *> to work on
 * behalf of the C++ std::string.  Feel free to ignore.
 */

#ifndef _hash_string_
#define _hash_string_

#ifdef __APPLE__
#include <ext/hash_map>
#else
#include <hash_map>
#endif
#include "error.h"

/**
 * Don't worry about this unless you want to.  The GNU extensions
 * to the C++ spec define a struct hash template that works for
 * all primitive types, but for reasons that aren't entirely
 * clear to me, they don't define or specialize the struct hash template
 * to work on the std::string class.  My guess is that the primitives like
 * int and const char * are considered core, whereas classes like string
 * are technically library, and that distinction was enough to prompt those
 * designing the struct hash built-in to omit a specialization on the string.
 * 
 * The following gobbledygook is a specialization of the hash struct inside
 * the __gnu_cxx template that adapts the const char * (e.g. C string) hasher
 * to work for std::strings (e.g. C++ strings).
 */

namespace __gnu_cxx {
    template<>
    struct hash<std::string> {
        size_t operator()(const std::string& x) const {
            return cstrhash(x.c_str());
        }
        hash<const char *> cstrhash;
    };
}

#endif
